Class: Streamdal::WasmFunction
- Inherits:
-
Object
- Object
- Streamdal::WasmFunction
- Defined in:
- lib/streamdal.rb
Overview
Data class to hold instantiated wasm functions
Instance Attribute Summary collapse
-
#instance ⇒ Object
Instance of an initialized wasm module and associated memory store.
-
#store ⇒ Object
Instance of an initialized wasm module and associated memory store.
Instance Method Summary collapse
-
#initialize ⇒ WasmFunction
constructor
A new instance of WasmFunction.
Constructor Details
#initialize ⇒ WasmFunction
Returns a new instance of WasmFunction.
48 49 50 51 |
# File 'lib/streamdal.rb', line 48 def initialize @instance = nil @store = nil end |
Instance Attribute Details
#instance ⇒ Object
Instance of an initialized wasm module and associated memory store
46 47 48 |
# File 'lib/streamdal.rb', line 46 def instance @instance end |
#store ⇒ Object
Instance of an initialized wasm module and associated memory store
46 47 48 |
# File 'lib/streamdal.rb', line 46 def store @store end |